{
Simple 1-character demo

This example uses an 8x8 character, 4 color text driver
to draw a 16x16 map on the screen and a simple 8x16 character that can move around
using the keyboard/gamepad.

A similar design could be accomplished with other Propeller/Hydra display drivers.

If you look at the Font_LinkDemo.spin and accompanying Font_LinkDemo.bmp, you will be
able to see the 'font' definition used in the demo.  Font_LinkDemo.spin was generated
with the FontGenerator utility (see: http://forums.parallax.com/forums/default.aspx?f=33&m=377666),
using the O (ORE_Text) option.

The DAT section (at the bottom of the listing) contains the map data used in the game.

In the VAR section, you will see that the character is managed with 4 variables (direction,
animation step, and x/y location).

The actual game loop does the following:
  1) Draw the map (background)          
  2) Check for gamepad events
  3) Draw the character 
  4) Update the display with changes
}

CON
  ' Proto/Demo Board setup
   _clkmode  = xtal1 + pll16x
   _xinfreq  = 5_000_000

   TV_BASE_PIN = 12
   KEYSPIN   = 26

{   
  ' Hydra setup
  _CLKMODE = xtal1 + pll8x
  _XINFREQ = 10_000_000 + 0000

   TV_BASE_PIN = 24 
   KEYSPIN  = 13   
}

{
' Hybrid Boards
   _clkmode  = xtal1 + pll16x
   _xinfreq  = 6_000_000
   
   TV_BASE_PIN = 24    
   KEYSPIN  = 12 
}

'' gamepad pin configuration (adjust for your setup)
  JOY_CLK = 16
  JOY_LCH = 17
  JOY_DATAOUT0 = 18
  JOY_DATAOUT1 = 19  
  
  ' NES bit encodings for NES gamepad 0
  NES0_RIGHT    = %00000000_00000001
  NES0_LEFT     = %00000000_00000010
  NES0_DOWN     = %00000000_00000100
  NES0_UP       = %00000000_00001000
  NES0_START    = %00000000_00010000
  NES0_SELECT   = %00000000_00100000
  NES0_B        = %00000000_01000000
  NES0_A        = %00000000_10000000  

  COLOR_BLACK  = $02
  COLOR_GREY   = $04
  COLOR_BROWN  = $5d
  COLOR_YELLOW = $6e
  COLOR_WHITE  = $07
  COLOR_GREEN2 = $ab
  COLOR_GREEN  = $9e
  COLOR_RED    = $4b
  COLOR_PURPLE = $2b
  COLOR_BLUE   = $fb
  COLOR_CYAN   = $58

  'direction constants
  DIR_SOUTH = 0
  DIR_NORTH = 2
  DIR_WEST  = 4
  DIR_EAST  = 6

VAR
  word Nes_Pad

  byte player_dir      'the player direction
  byte player_step     'the step the player is on
  byte player_x        'x position
  byte player_y        'y position

OBJ
  text : "ORE_TV_Text_008"  'the TV display driver
  key  : "comboKeyboard" 
  
PUB start 

    'start the tv text driver
    text.start(TV_BASE_PIN) 
    key.start(KEYSPIN)  
    player_dir  := DIR_SOUTH
    player_step := 0
    player_x    := 9
    player_y    := 8
    
    'set up the game loop
    repeat
    
      'Draw the map (background)          
      DrawMap
      
      'Check for gamepad events
      CheckGamePad

      'draw the player 
      DrawPlayer

      'build in a delay for animation
      waitcnt(5_000_000 + cnt)

      'update the screen with changes
      text.UpdateScreen

          
PRI DrawMap | ptr, x, y
    ptr := @mapinfo
    repeat x from 1 to 16
       repeat y from 1 to 16
          case byte[ptr]
            $60 :
             text.PokeChar(x,y,COLOR_GREEN, COLOR_GREY, COLOR_WHITE, byte[ptr])
                         
            $61 :
             text.PokeChar(x,y,COLOR_GREEN, COLOR_GREEN2, COLOR_BROWN, byte[ptr])                         
                         
            $64 :
             text.PokeChar(x,y,COLOR_GREEN, COLOR_GREY, COLOR_WHITE, byte[ptr])
          ptr++

PRI CheckGamePad
     Nes_Pad := NES_Read_Gamepad
     'gamepad has been moved down (south)               
     if (Nes_Pad & NES0_DOWN)
        player_dir := DIR_SOUTH
        
        'check to make sure the space the character is moving into
        'is clear
        if (CheckMove(player_x+1,player_y) == 0)
           player_x++
           if (player_x > 16)
              player_x := 16
           PlayerMove         

     'gamepad has been moved up (north)                
     if (Nes_Pad & NES0_UP)
        player_dir := DIR_NORTH
        
        'check to make sure the space the character is moving into
        'is clear
        if (CheckMove(player_x-1,player_y) == 0)
           player_x--
           if (player_x < 1)
              player_x := 1
           PlayerMove 

     'gamepad has been moved left (west)           
     if (Nes_Pad & NES0_LEFT)
        player_dir := DIR_WEST

        'check to make sure the space the character is moving into
        'is clear        
        if (CheckMove(player_x,player_y-1) == 0)
           player_y--
           if (player_y < 1)
              player_y := 1
           PlayerMove

     'gamepad has been moved right (east)                        
     if (Nes_Pad & NES0_RIGHT)
        player_dir := DIR_EAST

        'check to make sure the space the character is moving into
        'is clear        
        if (CheckMove(player_x,player_y+1) == 0)
           player_y++
           if (player_y > 16)
              player_y := 16           
           PlayerMove          

PRI CheckMove (move_x, move_y) | blocked
    'assume we are blocked, until we have checked
    'each space the character will occupy
    blocked := 1
    if (text.PeekChar(move_x -1, move_y) == $60)
       if (text.PeekChar(move_x, move_y) == $60)
          blocked := 0
    return blocked
          
PRI PlayerMove
    'advance the player step, and reset to 0 when necessary
    player_step++
    if (player_step > 1)
       player_step := 0
                      
PRI DrawPlayer | offset
    offset := player_dir + player_step
    text.PokeChar(player_x - 1, player_y, COLOR_GREEN, COLOR_BLUE,COLOR_BROWN,$02 + offset)
    text.PokeChar(player_x, player_y ,COLOR_GREEN, COLOR_RED, COLOR_BROWN,$22 + offset)
     
PRI NES_Read_Gamepad : nes_bits        |  i
' //////////////////////////////////////////////////////////////////
' NES Game Paddle Read
' //////////////////////////////////////////////////////////////////       
' reads both gamepads in parallel encodes 8-bits for each in format
' right game pad #1 [15..8] : left game pad #0 [7..0]
'
' set I/O ports to proper direction
' P3 = JOY_CLK      (4)
' P4 = JOY_SH/LDn   (5) 
' P5 = JOY_DATAOUT0 (6)
' P6 = JOY_DATAOUT1 (7)
' NES Bit Encoding
'
' RIGHT  = %00000001
' LEFT   = %00000010
' DOWN   = %00000100
' UP     = %00001000
' START  = %00010000
' SELECT = %00100000
' B      = %01000000
' A      = %10000000

' step 1: set I/Os
DIRA [JOY_CLK] := 1 ' output
DIRA [JOY_LCH] := 1 ' output
DIRA [JOY_DATAOUT0] := 0 ' input
DIRA [JOY_DATAOUT1] := 0 ' input

' step 2: set clock and latch to 0
OUTA [JOY_CLK] := 0 ' JOY_CLK = 0
OUTA [JOY_LCH] := 0 ' JOY_SH/LDn = 0
'Delay(1)

' step 3: set latch to 1
OUTA [JOY_LCH] := 1 ' JOY_SH/LDn = 1
'Delay(1)                            

' step 4: set latch to 0
OUTA [JOY_LCH] := 0 ' JOY_SH/LDn = 0

' data is now ready to shift out, clear storage
nes_bits := 0

' step 5: read 8 bits, 1st bits are already latched and ready, simply save and clock remaining bits
repeat i from 0 to 7

 nes_bits := (nes_bits << 1)
 nes_bits := nes_bits | INA[JOY_DATAOUT0] | (INA[JOY_DATAOUT1] << 8)

 OUTA [JOY_CLK] := 1 ' JOY_CLK = 1
 'Delay(1)             
 OUTA [JOY_CLK] := 0 ' JOY_CLK = 0
 
 'Delay(1)             
' invert bits to make positive logic
nes_bits := (!nes_bits & $FFFF)

' Keyboard (mapped onto NES buttons)
    if(key.keystate($C2))       'Up Arrow
      nes_bits|=NES0_UP
    if(key.keystate($C3))       'Down Arrow
      nes_bits|=NES0_DOWN
    if(key.keystate($C0))       'Left Arrow
      nes_bits|=NES0_LEFT
    if(key.keystate($C1))       'Right Arrow
      nes_bits|=NES0_RIGHT
    if(key.keystate($0D))       'Enter
      nes_bits|=NES0_START
    if(key.keystate(" "))       'Space
      nes_bits|=NES0_B

' //////////////////////////////////////////////////////////////////
' End NES Game Paddle Read
' //////////////////////////////////////////////////////////////////             
DAT
'Map information: a = forest / d = mountian / {`} = open (grass) 

mapinfo byte "dddddddddddddddd"
        byte "dddddd````dddddd" 
        byte "ddd``````````ddd" 
        byte "dd````````````dd" 
        byte "dd````````````dd" 
        byte "d``````````````d" 
        byte "d``````````````d" 
        byte "d``````````````d" 
        byte "d``````````````d" 
        byte "d``````````````d" 
        byte "d``````````````d" 
        byte "aa````````````dd" 
        byte "aa````````````dd" 
        byte "aaa``````````ddd" 
        byte "aaaaaa````dddddd" 
        byte "aaaaaaaadddddddd" 